import { AcceptedZipSources, BaseArchive } from './StreamArchive';
import { BasicArchiveInfo, PartialTweetGDPR, PartialTweet, AccountGDPR, ProfileGDPR, ClassicPayloadDetails, TwitterUserDetails, DMFile, DirectMessage, ArchiveSyntheticInfo, ExtendedGDPRInfo } from './TwitterTypes';
import DMArchive from './DMArchive';
import { EventTarget } from 'event-target-shim';
import TweetArchive from './TweetArchive';
export declare type ArchiveReadState = "idle" | "reading" | "indexing" | "tweet_read" | "user_read" | "dm_read" | "extended_read" | "ready";
declare type TwitterArchiveEvents = {
    zipready: CustomEvent<void>;
    userinfosready: CustomEvent<void>;
    tweetsread: CustomEvent<void>;
    indexready: CustomEvent<void>;
    willreaddm: CustomEvent<void>;
    willreadextended: CustomEvent<void>;
    ready: CustomEvent<void>;
    error: CustomEvent<any>;
};
declare type TwitterArchiveOnEvents = {
    onzipready: CustomEvent<void>;
    onuserinfosready: CustomEvent<void>;
    ontweetsread: CustomEvent<void>;
    onindexready: CustomEvent<void>;
    onwillreaddm: CustomEvent<void>;
    onwillreadextended: CustomEvent<void>;
    onready: CustomEvent<void>;
    onerror: CustomEvent<any>;
};
/**
 * Represents a full TwitterArchive. Support GDPR and classic archive.
 *
 * Tweets are available in `.tweets`.
 * Remember that, in searchs in particular, tweets are **NOT** sorted.
 *
 * Direct messages, parsed if archive is a GDPR archive, stored in `.messages`,
 * are returned and sorted from the most older to the more recent.
 */
export declare class TwitterArchive extends EventTarget<TwitterArchiveEvents, TwitterArchiveOnEvents> {
    protected load_images_in_zip: boolean;
    protected _ready: Promise<void>;
    protected archive: BaseArchive<any>;
    /** Current archive load state. */
    state: ArchiveReadState;
    /**
     * Access to extended infos. See `ExtendedGDPRInfo` interface.
     *
     * Defined only if `.is_gdpr === true`.
     * */
    extended_gdpr: ExtendedGDPRInfo;
    protected _info: BasicArchiveInfo;
    protected statuses: TweetArchive;
    protected dms: DMArchive;
    protected dm_img_archive: BaseArchive<any>;
    protected dm_img_group_archive: BaseArchive<any>;
    protected _is_gdpr: boolean;
    /**
     * Build a new instance of TwitterArchive.
     *
     * @param file Accept any source that `JSZip` library accepts, and string for filenames.
     * If this parameter is `null`, then you'll need to load each part one by one !
     * @param build_extended True if `.extended_gdpr` should be built (only if **file** is a GDPR archive.)
     * @param keep_loaded If possible, free the memory after load if set to false.
     * @param load_images_in_zip In Twitter GDPR archives v2, tweet and dm images are in ZIP archives inside the ZIP.
     * If `true`, TwitterArchive will extract its content in RAM to allow the usage of images.
     * If `false`, DMs images will be unavailable.
     * If `undefined`, Twitter will extract in RAM in browser mode, and leave the ZIP untouched in Node.js.
     *
     * If you want to save memory, set this parameter to `false`,
     * and before using `.dmImage()` methods, check if you need to load DM images ZIP
     * with `.requiresDmImageZipLoad()`.
     *
     * Then, if you need to, load the DM image ZIP present in the archive using `.loadCurrentDmImageZip()`.
     * **Please note that `keep_loaded` should be set to `true` to use this method !**
     */
    constructor(file: AcceptedZipSources | Promise<AcceptedZipSources> | null, build_extended?: boolean, keep_loaded?: boolean, load_images_in_zip?: boolean);
    protected initGDPR(extended: boolean, keep_loaded: boolean): Promise<void>;
    protected initExtendedGDPR(): Promise<void>;
    protected initClassic(): Promise<void>;
    protected isGDPRArchive(): boolean;
    /** --------------- */
    /** DM IMAGES STUFF */
    /** --------------- */
    /**
     * Give the media url in direct message, obtain the Blob-bed image.
     * For use in Node.js, you must set `as_array_buffer` to `true` !
     */
    dmImageFromUrl(url: string, is_group?: boolean, as_array_buffer?: boolean): Promise<ArrayBuffer | Blob>;
    /**
     * Extract a direct message image from GDPR archive (exact filename required).
     * For use in Node.js, you must set `as_array_buffer` to `true` !
     */
    dmImage(name: string, is_group?: boolean, as_array_buffer?: boolean): Promise<Blob | ArrayBuffer>;
    /**
     * Return all the images of a direct message, as blob or array buffer.
     *
     * If the message does not exists or the DM archive is not loaded / available,
     * return an empty array.
     *
     * Otherwise, return a array of `Blob` / `ArrayBuffer`
     *
     * @param direct_message Direct Message id | Direct message
     * @param as_array_buffer Return an `ArrayBuffer` array, instead of a `Blob` array
     */
    dmImagesOf(direct_message: string | DirectMessage, as_array_buffer?: boolean): Promise<(Blob | ArrayBuffer)[]>;
    /** ------------------------------------ */
    /** TWEET RELATED STUFF THAT SHOULD MOVE */
    /** ------------------------------------ */
    /**
     * @deprecated Use `TweetArchive.isGDPRTweet()` instead.
     */
    isGDPRTweet(tweet: PartialTweetGDPR | PartialTweet): tweet is PartialTweetGDPR;
    /**
     * @deprecated Use `.tweets.month()` instead.
     */
    month(month: string, year: string): PartialTweet[];
    /**
     * @deprecated Use `.tweets.fromThatDay()` instead.
     */
    fromThatDay(): PartialTweet[];
    /**
     * @deprecated Use `.tweets.between()` instead.
     */
    between(since: Date, until: Date): PartialTweet[];
    /**
     * @deprecated Please use `.tweets.single()` instead.
     */
    id(id_str: string): PartialTweet | null;
    /**
     * @deprecated Please use `.tweets.all` instead.
     */
    get all(): PartialTweet[];
    /**
     * @deprecated Will be removed. Tweet index is moved to `.tweets.index` and
     * archive info to `.info`.
     */
    get index(): {
        info: TwitterUserDetails;
        archive: {
            created_at: string;
            tweets: number;
        };
        by_id: import("./TwitterTypes").TweetIndex;
        years: {
            [year: string]: {
                [month: string]: import("./TwitterTypes").TweetIndex;
            };
        };
    };
    /**
     * @deprecated Please use `.tweets.length` instead.
     */
    get length(): number;
    /** ----------------- */
    /** ARCHIVE ACCESSORS */
    /** ----------------- */
    /**
     * `true` if you need to load a DM image ZIP in order to use `.dmImage()`.
     *
     * If you need to, use the `.loadArchivePart()` method:
     * - Loading the current DM image ZIP (if you've the constructor to not do so)
     * ```ts
     * await archive.loadArchivePart({ current_dm_images: true });
     * ```
     *
     * ---
     *
     * - Loading a custom DM ZIP (from external file)
     * ```ts
     * await archive.loadArchivePart({ dm_image_file: '<filepath.zip>' });
     * ```
     */
    get requires_dm_image_load(): boolean;
    /** Access to the `DMArchive` object. Will be undefined if `.is_gdpr === false`. */
    get messages(): DMArchive;
    /**
     * ID of the user who created this archive.
     *
     * Shortcut of `.info.user.id`.
     */
    get owner(): string;
    /** Access to the `TweetArchive` object. Contains all the tweets of this archive. */
    get tweets(): TweetArchive;
    /**
     * Screen name (@) of the user who created this archive.
     * May be obsolete (user can change screen_name over time).
     *
     * Shortcut of `.info.user.screen_name`.
     */
    get owner_screen_name(): string;
    /** Archive creation date. Not accurate in GDPR archive (will be the current date). */
    get generation_date(): Date;
    /**
     * Archive information.
     *
     * - `.info.archive` : `{ created_at: string, tweets: number }`
     * - `.info.user`: See `TwitterUserDetails`
     */
    get info(): BasicArchiveInfo;
    /** True if archive is a GDPR archive. */
    get is_gdpr(): boolean;
    /**
     * Raw archive object. Can be used to get specific files.
     *
     * Returns `[twitter_archive, dm_image_archive, dm_group_image_archive]`.
     */
    get raw(): [BaseArchive<any>, BaseArchive<any> | undefined, BaseArchive<any> | undefined];
    protected get should_autoload_zip_img(): boolean;
    /** Resolved when archive read is over. */
    ready(): Promise<void>;
    /** -------------------- */
    /** SIDELOADING MANUALLY */
    /** -------------------- */
    /**
     * Load a part of a GDPR archive.
     *
     * Set current archive as GDPR archive.
     *
     * ---
     *
     * **Warning**: The DMs image parameters causes the read of a new archive.
     * Archive read is asynchronous, you **must** wait read end before trying to get images.
     */
    loadArchivePart(parts?: {
        tweets?: PartialTweetGDPR[];
        account?: AccountGDPR;
        profile?: ProfileGDPR;
        dms?: DMFile[];
        dm_image_file?: AcceptedZipSources | Promise<AcceptedZipSources>;
        dm_image_group_file?: AcceptedZipSources | Promise<AcceptedZipSources>;
        current_dm_images?: boolean;
    }): Promise<void>;
    /**
     * Load a part of a classic archive.
     *
     * Set current archive as non-gdpr archive.
     */
    loadClassicArchivePart(parts?: {
        tweets?: PartialTweet[];
        payload?: ClassicPayloadDetails;
        user?: TwitterUserDetails;
    }): void;
    /**
     * Convert GDPR tweets into classic tweets, then register them into the index.
     */
    protected readGDPRTweets(tweets: PartialTweetGDPR[]): void;
    /**
     * Register tweets into the index.
     */
    protected readTweets(tweets: PartialTweet[]): void;
    /**
    * Import a custom ZIP file as DM single-conversation images file.
    */
    protected importDmImageZip(file: AcceptedZipSources | Promise<AcceptedZipSources>): Promise<void>;
    /**
     * Import a custom ZIP file as DM group-conversation images file.
     */
    protected importDmGroupImageZip(file: AcceptedZipSources | Promise<AcceptedZipSources>): Promise<void>;
    /**
     * Load/reload zip archives that contains DM images,
     * if `load_images_in_zip` parameter was set to false.
     *
     * You need to have the archive loaded to accomplish this action
     * (constructor `keep_loaded` parameter should be set to `true`).
     *
     * Note that if any zip is found in the archive, this method will just do nothing.
     */
    protected loadCurrentDmImageZip(): Promise<void>;
    /** --------------------------------------- */
    /** ARCHIVE FINGERPRINTING AND INFORMATIONS */
    /** --------------------------------------- */
    /**
     * Résumé of this archive.
     *
     * Contains the 'archive index' (without the tweets),
     * if the archive is GDPR, last tweet date,
     * tweet count and dm count.
     */
    get synthetic_info(): ArchiveSyntheticInfo;
    /**
     * `.synthetic_info`, but without hash. Used to... calculate hash.
     */
    protected get synthetic_info_without_hash(): ArchiveSyntheticInfo;
    /**
     * Create a pseudo-unique hash from a archive information (number of tweets, owner, last tweet date...),
     * in order to have a fingerprint of one archive.
     */
    static hash(from: ArchiveSyntheticInfo | TwitterArchive): string;
    /**
     * Pseudo-unique hash from the archive information (number of tweets, owner, last tweet date...),
     * in order to have a fingerprint of this archive.
     */
    get hash(): string;
}
export {};
