import { PartialTweet, TweetIndex, PartialTweetGDPR, PartialTweetUser } from "./TwitterTypes";
export declare class TweetArchive {
    protected by_id: TweetIndex;
    protected years: {
        [year: string]: {
            [month: string]: TweetIndex;
        };
    };
    protected user_cache: PartialTweetUser;
    /** ------------------ */
    /** ARCHIVE MANAGEMENT */
    /** ------------------ */
    /**
     * Add tweets to this archive.
     */
    add(tweets: PartialTweet[]): void;
    /**
     * Add unconverted GDPR tweets to this archive.
     */
    addGDPR(tweets: PartialTweetGDPR[]): void;
    /** ---------------- */
    /** TWEET NAVIGATION */
    /** ---------------- */
    /** Extract tweets from a specific month. */
    month(month: string, year: string): PartialTweet[];
    /** Find tweets made on the same day (= month, = day), but in all years. */
    fromThatDay(): PartialTweet[];
    /** Get tweets in a specific time interval. */
    between(since: Date, until: Date): PartialTweet[];
    /** Get a single tweet by ID. Returns `null` if tweet does not exists. */
    single(id_str: string): PartialTweet | null;
    /** --------- */
    /** ACCESSORS */
    /** --------- */
    /**
     * Index of tweets by years.
     *
     * Example, get index of tweets posted on 2019/01 :
     * **<index>.years[2019][1]**
     */
    get index(): {
        [year: string]: {
            [month: string]: TweetIndex;
        };
    };
    /**
     * Index of tweets by ID.
     */
    get id_index(): TweetIndex;
    /** Number of tweets in this archive. */
    get length(): number;
    /** All tweets registered in this archive. */
    get all(): PartialTweet[];
    /** --------- */
    /** ITERATORS */
    /** --------- */
    /**
     * Iterate through the tweets
     */
    [Symbol.iterator](): Generator<PartialTweet, void, undefined>;
    /**
     * Iterate through tweets, sorted by year and month.
     *
     * At each iteration: [year: string, month: string, tweet: PartialTweet]
     */
    monthIterator(): Generator<[string, string, PartialTweet], void, void>;
    /** ------- */
    /** HELPERS */
    /** ------- */
    /**
     * Needed to convert GDPR tweets to PartialTweets.
     *
     * **You should not use this method! This is for internal purpose.**
     */
    __initUserCache(user_info: {
        id_str: string;
        name: string;
        screen_name: string;
        profile_image_url_https: string;
    }): void;
    protected convertToPartial(tweet: PartialTweetGDPR): PartialTweet;
    /**
     * True if given tweet is coming from a GDPR archive.
     *
     * Tweets getted by available getters are NOT GDPR tweets, they've been converted !
     */
    static isGDPRTweet(tweet: PartialTweetGDPR | PartialTweet): tweet is PartialTweetGDPR;
    /** Return the `Date` object affiliated to **tweet**. */
    static dateFromTweet(tweet: PartialTweet): Date;
    /**
     * Parse a raw Twitter date, from a `dm.createdAt` or `tweet.created_at`.
     *
     * For a tweet, please use `dateFromTweet(tweet)` instead, it's optimized !
     */
    static parseTwitterDate(date: string): Date;
    /**
     * Return true if **tweet** contains media(s).
     *
     * This includes photos, videos or animated GIF.
     */
    static isWithMedia(tweet: PartialTweet): boolean;
    /**
     * Return true if **tweet** contains a video or one animated GIF.
     *
     * Twitter's GIF are mp4 encoded.
     */
    static isWithVideo(tweet: PartialTweet): boolean;
    /**
     * Sort tweets by ID.
     */
    static sortTweets(tweets: PartialTweet[]): PartialTweet[];
}
export default TweetArchive;
